package cn.demo.biz;

import cn.demo.biz.dto.*;
import cn.demo.orm.dao.*;
import cn.demo.orm.po.*;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Future;
import java.util.stream.Collectors;

@Slf4j
@Service
public class AsyncService {

    @Resource
    private TopicManageMapper topicManageMapper;

    @Resource
    private AppMetaMapper appMetaMapper;

    @Resource
    private PointMetaMapper pointMetaMapper;
    @Resource
    private TopicMetaMapper topicMetaMapper;
    @Resource
    private AppBomMapper appBomMapper;
    @Resource
    private PointInstMapper pointInstMapper;
    @Resource
    private TopicMsgAllMapper topicMsgAllMapper;

    @Async("alterValuAsyncPool")
    public void refreshPointInstValue(TopicMsgAll topicMsg){
        List<TopicManage> topicManages = topicManageMapper.listByWhere(TopicManage.builder()
                .topic(topicMsg.getTopic())
                .build());
        if (topicManages.isEmpty()){
            return;
        }
        TopicManage topicManage = topicManages.get(0);
        Long pointInstId = topicManage.getPointInstId();

        PointInst pointInst = new PointInst();
        pointInst.setId(pointInstId);
        pointInst.setValu(topicMsg.getValu());
        pointInst.setTs(System.currentTimeMillis());
        pointInstMapper.updateBatch(ListUtil.toList(pointInst));

        //处理完成 flag设置0
        topicMsgAllMapper.updateHandleFlagById(topicMsg.getId());
        log.info("topic:{}点位值更新完毕",topicMsg.getTopic());
    }


    @Async("myAsyncPool")
    public Future<List<InstPointDTO>> getAppPointInstInfo(Long id) {
        List<InstPointDTO> instPointList = new ArrayList<>();

        List<PointInst> pointInsts = pointInstMapper.listByWhere(PointInst.builder()
                .appInstId(id)
                .build());
        if (!pointInsts.isEmpty()) {
            for (PointInst item : pointInsts) {
                InstPointDTO instPoint = new InstPointDTO();
                BeanUtil.copyProperties(item,instPoint);
                //remark, unit
                PointMeta oneById = pointMetaMapper.getOneById(item.getPointMetaId());
                instPoint.setRemark(oneById.getRemark());
                instPoint.setUnit(oneById.getUnit());
                instPointList.add(instPoint);
            }
        }

        return new AsyncResult<>(instPointList);
    }

    @Async("myAsyncPool")
    public void handleAppInstExtra(Long appMetaId, InstDTO instDTO) {
        List<AppMeta> appMetas = appMetaMapper.listByWhere(AppMeta.builder()
                .id(appMetaId)
                .build());
        AppMeta appMeta = appMetas.get(0);
        instDTO.setRemark(appMeta.getRemark());
        instDTO.setSnt(appMeta.getSnt());
    }

    @Async("myAsyncPool")
    public void generateTopicInstList(List<PointInst> pointInsts, BomPointDTO bomPoint, Long appInstId) {
        List<TopicManage> preTopicList = new ArrayList<>();
        for (PointInst pointInstItem : pointInsts) {
            List<BomTopicDTO> bomTopicList = bomPoint.getBomTopicList();
            for (BomTopicDTO bomTopic : bomTopicList) {
                String resTopic = StrUtil.replace(bomTopic.getTopic(), "[serno]", pointInstItem.getSerno());
                TopicManage topicInst = TopicManage.builder()
                        .topic(resTopic)
                        .exampleData(bomTopic.getExampleData())
                        .remark("[" + pointInstItem.getSerno() + "]---" + bomTopic.getRemark())
                        .directType(bomTopic.getDirectType())
                        .createTs(System.currentTimeMillis())
                        .enableFlag(0)
                        .pointInstId(pointInstItem.getId())
                        .appInstId(appInstId)
                        .build();
                preTopicList.add(topicInst);
            }
        }
        topicManageMapper.insertBatch(preTopicList);
    }


    @Async("myAsyncPool")
    public void saveAppBom(AppBom req) {
        appBomMapper.insertBatch(ListUtil.toList(req));
    }

    @Async("myAsyncPool")
    public void saveTopicMetas(List<BomTopicDTO> bomTopicList, Long pointMetaId) {
        List<BomTopicDTO> collect = bomTopicList.stream().distinct().collect(Collectors.toList());
        List<TopicMeta> topicMetas = new ArrayList<>();
        for (BomTopicDTO item : collect) {
            TopicMeta build1 = TopicMeta.builder()
                    .pointMetaId(pointMetaId)
                    .createTs(System.currentTimeMillis())
                    .directType(item.getDirectType())
                    .exampleData(item.getExampleData())
                    .remark(item.getRemark())
                    .topic(item.getTopic())
                    .build();
            topicMetas.add(build1);
        }
        topicMetaMapper.insertBatch(topicMetas);
    }

    @Async("myAsyncPool")
    public Future<BomDTO> getAppMetaInfo(Long id) {
        List<AppMeta> appMetas = appMetaMapper.listByWhere(AppMeta.builder()
                .id(id)
                .build());
        AppMeta appMeta = appMetas.get(0);
        //1级
        BomDTO bom = new BomDTO();
        bom.setId(appMeta.getId())
                .setName(appMeta.getName())
                .setRemark(appMeta.getRemark())
                .setSnt(appMeta.getSnt())
                .setTs(appMeta.getTs());

        return new AsyncResult<>(bom);
    }

    @Async("myAsyncPool")
    public Future<BomDTO> getPointTopicInfo(Long id) {
        BomDTO bom = new BomDTO();
        //2级
        List<BomPointDTO> bomPointList = new ArrayList<>();
        List<AppBom> appBoms = appBomMapper.listByWhere(AppBom.builder()
                .appMetaId(id)
                .build());
        for (AppBom apBom : appBoms) {
            List<PointMeta> pointMetas = pointMetaMapper.listByWhere(PointMeta.builder()
                    .id(apBom.getPointMetaId())
                    .build());
            PointMeta pointMeta = pointMetas.get(0);
            BomPointDTO build = new BomPointDTO();
            BeanUtil.copyProperties(pointMeta, build);
            bomPointList.add(build);
        }
        bom.setBomPointList(bomPointList);


        //3级
        List<BomPointDTO> bomPoints3 = new ArrayList<>();
        for (BomPointDTO point : bomPointList) {
            Long id1 = point.getId();
            List<BomTopicDTO> bomTopicList = new ArrayList<>();
            List<TopicMeta> topicMetas = topicMetaMapper.listByWhere(TopicMeta.builder()
                    .pointMetaId(id1)
                    .build());

            for (TopicMeta topicMeta : topicMetas) {
                BomTopicDTO pre = new BomTopicDTO();
                BeanUtil.copyProperties(topicMeta, pre);
                bomTopicList.add(pre);
            }

            point.setBomTopicList(bomTopicList);
            bomPoints3.add(point);
        }
        bom.setBomPointList(bomPoints3);

        return new AsyncResult<>(bom);
    }
}
