package cn.demo.jms.sub;

import cn.demo.MsgObj;
import cn.demo.TestObj;
import cn.demo.biz.AsyncService;
import cn.demo.orm.dao.TopicManageMapper;
import cn.demo.orm.dao.TopicMsgRecentMapper;
import cn.demo.orm.dao.TopicMsgAllMapper;
import cn.demo.orm.po.TopicManage;
import cn.demo.orm.po.TopicMsgRecent;
import cn.demo.orm.po.TopicMsgAll;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.jms.*;
import java.util.List;

/**
 * jms共有5种消息类型
 * TextMessage  字符串消息
 * ObjectMessage  对象消息
 * MapMessage  键值对消息
 * BytesMessage  连续字节流 消息
 * StreamMessage 流 消息--------artemis mq 不能发送StreamMessage
 */
@Slf4j
@Component
public class SubQueue {
    @Resource
    private TopicMsgRecentMapper topicMsgRecentMapper;
    @Resource
    private TopicMsgAllMapper topicMsgAllMapper;
    @Autowired
    private AsyncService asyncService;
    @Resource
    private TopicManageMapper topicManageMapper;

    @JmsListener(destination = "queue.testbytes")
    public void handleBytes(BytesMessage bytesMessage) throws JMSException {
        byte[] b = new byte[1024];
        int len = -1;
        while ((len = bytesMessage.readBytes(b)) != -1) {
            String s = new String(b, 0, len);
            log.info("订阅到的1024字节的字符串:{}",s);
        }
    }


    @JmsListener(destination = "queue.testmap")
    public void handleMap(MapMessage map) {
        try {
            String k1 = map.getString("k1");
            long k2 = map.getLong("k2");
            log.info("监听到queue.testmap里的键值对k1:{}", k1);
            log.info("监听到queue.testmap里的键值对k2:{}", k2);
        } catch (JMSException e) {
            e.printStackTrace();
        }
    }

    @JmsListener(destination = "queue.sample")
    public void handleMessage(TextMessage message) {
        try {
            String text = message.getText();
            log.info("监听到queue.sample里的信息:{}", text);

        } catch (JMSException e) {
            e.printStackTrace();
        }
    }

    @JmsListener(destination = "queue.testobj")
    public void handleObjMsg(ObjectMessage message) {
        try {
            TestObj object = (TestObj) message.getObject();
            log.info("监听到queue.testobj里的信息:{}", object);
        } catch (JMSException e) {
            e.printStackTrace();
            log.error("ObjectMessage转换失败!", e);
        }
    }


    @JmsListener(destination = "queue.msgObj",concurrency = "2")
    public void handleObj2(ObjectMessage message) {
        try {
            MsgObj object = (MsgObj) message.getObject();
            log.info("监听到queue.msgObj里的信息:{}", object);

            //不存在这个topic,不做业务
            List<TopicManage> topicManages = topicManageMapper.listByWhere(TopicManage.builder()
                    .topic(object.getKey())
                    .build());
            if (topicManages.isEmpty()){
                return;
            }





            //(有条件的，可以存入时序数据库如influx db ,然后从influx里拿点位实时数据，显示折线图/散点图等)


            //replace最新记录
            topicMsgRecentMapper.replaceIntoOneTopic(TopicMsgRecent.builder()
                    .ts(object.getTs())
                    .valu(object.getValue())
                    .topic(object.getKey())
                    .build());

            //save全量记录----业务记录用
            TopicMsgAll build = TopicMsgAll.builder()
                    .ts(object.getTs())
                    .valu(object.getValue())
                    .topic(object.getKey())
                    .handleFlag(1) //1-未处理;0-处理完成
                    .build();
            topicMsgAllMapper.insertOne(build);

            //处理业务
            doSomething(build);

        } catch (JMSException e) {
            e.printStackTrace();
            log.error("ObjectMessage转换失败!", e);
        }
    }


    private void doSomething(TopicMsgAll topicMsg){
        //刷新点位值
        asyncService.refreshPointInstValue(topicMsg);
        //温度.....告警...
        //门...打开/关闭了...
        //.................

        //  up/ctrl/[serno]
        //  0/1



        log.info("mqtt->jms订阅处理完成!");

    }
}
