package cn.demo.mqtt.thread;

import cn.demo.orm.dao.TopicManageMapper;
import cn.demo.orm.po.TopicManage;
import cn.hutool.core.util.StrUtil;
import org.fusesource.mqtt.client.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.jms.core.JmsTemplate;
import org.springframework.stereotype.Service;

import cn.demo.MsgObj;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ThreadPoolExecutor;

@Slf4j
@Service
public class MyService {


    @Autowired
    private JmsTemplate jmsTemplate;

    @Resource
    private TopicManageMapper topicManageMapper;


    private final Map<String, Thread> tMap = new ConcurrentHashMap<>();

    public void createThd(String tname) {
        Thread thread = new Thread(new MyRunnable(tname));
        thread.setName(tname);
        thread.start();
        tMap.put(tname, thread);
    }

    public void removeThd(String tName) {
        Thread thread = tMap.get(tName);
        if (thread!=null){
            thread.interrupt();
        }
    }

    public Set<String> listCurrentThd(){
        return tMap.keySet();
    }


    public void sendJmsObj(String topic, String v) {
        log.info("topic:{},valu:{}",topic,v);

        List<TopicManage> topicManages = topicManageMapper.listByWhere(TopicManage.builder()
                .topic(topic)
                .build());

        if (topicManages.isEmpty()){
            log.info("不符合系统topic实例清单的主题!");
            return;
        }

        for (TopicManage topicManage : topicManages) {
            if(topicManage.getEnableFlag()==0){
                MsgObj msgObj = MsgObj.builder()
                        .key(topic)
                        .value(v)
                        .ts(System.currentTimeMillis())
                        .build();
                jmsTemplate.convertAndSend("queue.msgObj", msgObj);
            }
        }
    }


}
