package cn.demo.rest;

import ch.qos.logback.classic.Level;
import ch.qos.logback.classic.LoggerContext;
import cn.demo.biz.AptBomService;
import cn.demo.biz.AptInstService;
import cn.demo.biz.AsyncService;
import cn.demo.biz.dto.BomDTO;
import cn.demo.biz.dto.BomPointDTO;
import cn.demo.biz.dto.InstDTO;
import cn.demo.biz.group.AddAppMetaGroup;
import cn.demo.biz.group.AddBomPointTopicGroup;
import cn.demo.mqtt.send.SendMqttService;
import cn.demo.mqtt.thread.MyService;
import cn.demo.orm.dao.*;
import cn.demo.orm.po.*;
import cn.demo.rest.req.AddAptInstReq;
import cn.demo.rest.req.SendCmdReq;
import cn.demo.rest.req.TopicDataOverTimeReq;
import cn.demo.sys.JacksonUtil;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.date.StopWatch;
import cn.hutool.core.util.StrUtil;
import cn.hutool.system.JvmInfo;
import cn.hutool.system.SystemUtil;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.concurrent.ExecutionException;

@Slf4j
@RestController
public class CloudApi {

    @Autowired
    private AptBomService aptBomService;
    @Resource
    private AppMetaMapper appMetaMapper;
    @Resource
    private PointMetaMapper pointMetaMapper;

    @Autowired
    private AsyncService asyncService;
    @Autowired
    private SendMqttService sendMqttService;
    @Resource
    private TopicMsgRecentMapper topicMsgRecentMapper;
    @Resource
    private PointInstMapper pointInstMapper;
    @Resource
    private AppInstMapper appInstMapper;

    @Autowired
    private AptInstService aptInstService;

    @Resource
    private TopicMsgAllMapper msgAllMapper;

    @GetMapping("/third/jvmInfo")
    public JvmInfo getInfo() throws Exception {
        return SystemUtil.getJvmInfo();
    }

    @Autowired
    private MyService myService;

    @GetMapping("/testSubTopic")
    public String tempSubscribedOneTopic(String topic){
//        myService.removeThd(topic);
        log.info("临时订阅:{}",topic);
        myService.createThd(topic);
        return "ok";
    }

    @GetMapping("/cancelTempSubTopic")
    public String cancelTempSubTopic(String topic){
        myService.removeThd(topic);
        return "ok";
    }

    @GetMapping("/getCurrentSubTopics")
    public Set<String>  getCurrentSubTopics(){
        return myService.listCurrentThd();
    }


    /**
     * web端,移动端 下方cmd
     * @param req
     * @return
     * @throws Exception
     */
    @PostMapping("/down/sendCmd")
    public String sendCmd(@RequestBody SendCmdReq req) throws Exception {
        sendMqttService.send(req.getTopic(),
                req.getValu(),
                req.getQos());
        return "ok";
    }

    /**
     * topic最新值
     * @param topic
     * @return
     */
    @GetMapping("/getTopicRecentValu")
    public String getTopicRecentValu(@RequestParam String topic) {
        return topicMsgRecentMapper.getTopicRecentValu(topic);
    }

    /**
     * topic一段时间内的值集合
     * @param req
     * @return
     */
    @PostMapping("/findTopicDataOverTime")
    public List<TopicMsgAll>  findTopicDataOverTime(@Validated @RequestBody
                                                        TopicDataOverTimeReq req){
        return msgAllMapper.listByWhere(TopicMsgAll.builder()
                .startTime(req.getStartTime())
                .endTime(req.getEndTime())
                .topic(req.getTopic())
                .build());
    }


    /**
     * 查询bom清单
     * @param id
     * @return
     * @throws ExecutionException
     * @throws InterruptedException
     */
    @GetMapping("/getBomByAppMetaId")
    public BomDTO getOneBomByAppMetaId(Long id) throws ExecutionException, InterruptedException {
        return aptBomService.getOneBomByAppMetaId(id);
    }


    @GetMapping("/getOneInstDTOById")
    public InstDTO getOneInstDTOById(Long id) throws ExecutionException, InterruptedException {
        return aptInstService.getAptInstByAppId(id);
    }




    /*
    {
	"name":"rty34y",
	"remark":"tyru45u",
	"snt":"jkrtu"
}
-----------
{
    "code": 0,
    "msg": "操作成功",
    "content": {
        "id": 5,
        "name": "rty34y",
        "remark": "tyru45u",
        "ts": 1688004833422,
        "snt": "jkrtu"
    },
    "timestamp": 1688004833605,
    "traceDateTime": "2023-06-29 10:13:53"
}
     */
    @PostMapping("/createAppMeta")
    public AppMeta createAppMeta(@RequestBody @Validated({AddAppMetaGroup.class})
                                         AppMeta req) {
        AppMeta icd = new AppMeta();
        BeanUtil.copyProperties(req, icd);
        icd.setTs(System.currentTimeMillis());
        appMetaMapper.insertOne(icd);
        return icd;
    }

    @GetMapping("/findAppMetaList")
    public List<AppMeta>  findAppMetaList(){
        return appMetaMapper.listByWhere(new AppMeta());
    }

//--------------------------------------------------------------------
    /*
http://localhost:8627/iotcenter/alterLogLevel/info
     */
    @GetMapping("/alterSysLogLevel/{level}")
    public String alterSysLogLevel(@PathVariable String level){
        LoggerContext context = (LoggerContext) LoggerFactory.getILoggerFactory();
        context.getLogger("ROOT").setLevel(Level.valueOf(level));
        return "ok";
    }


    @GetMapping("/alterPkgLogLevel")
    public String alterPkgLogLevel(@RequestParam String level, @RequestParam String pkgName){
        LoggerContext context = (LoggerContext) LoggerFactory.getILoggerFactory();
        context.getLogger(pkgName).setLevel(Level.valueOf(level));
        return "ok";
    }

//--------------------------------------------------------------------




    @GetMapping("/t1")
    public String te(){
        //logback日志级别分为5种，分别是：
        //
        //TRACE：运行堆栈信息，使用率低
        //DEBUG：程序员调试代码使用
        //INFO：记录运维过程数据
        //WARN：记录运维过程报警数据
        //ERROR：记录错误堆栈信息

        log.trace("trace---test!!!!!");
        log.debug("debug---test!!!!!");
        log.info("info---test!!!!!");
        log.warn("warn---test!!!!!");
        log.error("error---test!!!!!");

        return "ok";
    }











    /*
{
    "appMetaId": 5,
    "cost": 1,
    "name": "sd",
    "remark": "湿度传感器",
    "ser": "sd",
    "unit": "%",
    "bomTopicList": [
        {
            "topic": "up/sd/valu/[serno]",
            "exampleData": 51,
            "remark": "湿度值范围0-100",
            "directType": "up"
        }
    ]
}
----
{
    "code": 0,
    "msg": "操作成功",
    "content": "ok",
    "timestamp": 1688006351319,
    "traceDateTime": "2023-06-29 10:39:11"
}
     */
    @PostMapping(value = "/createBomPointTopic", produces = {MediaType.APPLICATION_JSON_VALUE})
    public String createBomPointTopic(@RequestBody @Validated({AddBomPointTopicGroup.class})
                                              BomPointDTO req) {
        StopWatch sw = StopWatch.create("cm");
        sw.start();

        //1 save point_meta
        PointMeta pre1 = new PointMeta();
        BeanUtil.copyProperties(req, pre1);
        pre1.setTs(System.currentTimeMillis());
        pointMetaMapper.insertOne(pre1);
        Long pointMetaId = pre1.getId();


        //2.1 save app_bom
        Long appMetaId = req.getAppMetaId();
        AppBom build = AppBom.builder()
                .pointMetaId(pointMetaId)
                .appMetaId(appMetaId).build();
//        appBomMapper.insertBatch(ListUtil.toList(build));
        asyncService.saveAppBom(build);


        //2.2 save topic_meta
        asyncService.saveTopicMetas(req.getBomTopicList(), pointMetaId);


        sw.stop();
        log.info("耗时:{}", sw.getTotalTimeMillis());


        return "ok";
    }


    /*
{
    "appMetaId": 7,
    "appInstName": "新建测试1"
}
     */
    @PostMapping(value = "/createAptInst", produces = {MediaType.APPLICATION_JSON_VALUE})
    public String createAptInst(@RequestBody @Validated AddAptInstReq req) throws ExecutionException, InterruptedException {
        StopWatch sw = StopWatch.create("capt");
        sw.start();

        BomDTO oneBom = aptBomService.getOneBomByAppMetaId(req.getAppMetaId());
        System.out.println(JacksonUtil.obj2StringPretty(oneBom));

        //build appInst
        String defaultName = oneBom.getName() + System.currentTimeMillis();
        AppInst appInst = AppInst.builder()
                .sname(StrUtil.isBlank(req.getAppInstName()) ? defaultName : req.getAppInstName())
                .sno(oneBom.getSnt() + System.currentTimeMillis())
                .ts(System.currentTimeMillis())
                .appMetaId(oneBom.getId()).build();
        appInstMapper.insertBatch(ListUtil.toList(appInst));
        System.out.println(appInst);

        //build pointInst and topicInst
        List<BomPointDTO> bomPointList = oneBom.getBomPointList();
        for (BomPointDTO bomPoint : bomPointList) {
            generatePointInstItem(bomPoint, appInst.getId());
        }

        sw.stop();
        log.info("耗时:{}", sw.getTotalTimeMillis());
        return "ok";
    }


    private void generatePointInstItem(BomPointDTO bomPoint, Long appInstId) {

        List<PointInst> pointInsts = new ArrayList<>();
        for (long i = 0; i < bomPoint.getCost(); i++) {
            //save pointInst
            PointInst pointInst = PointInst.builder()
                    .appInstId(appInstId)
                    .pointMetaId(bomPoint.getId())
                    .serno(bomPoint.getSer() + i)
                    .ts(System.currentTimeMillis())
                    .build();
            pointInsts.add(pointInst);
        }
        pointInstMapper.insertBatch(pointInsts);
        asyncService.generateTopicInstList(pointInsts, bomPoint, appInstId);
    }


}
