//package cn.demo.sys.log;
//
//import cn.ath.knowwikibackend.async.biz.AsyncAllService;
//import cn.ath.knowwikibackend.es.IdGenerator;
//import cn.ath.knowwikibackend.es.model.OperationLogIndex;
//import cn.ath.knowwikibackend.util.Ip2regionUtil;
//import cn.hutool.core.date.DateTime;
//import cn.hutool.core.util.StrUtil;
//import lombok.extern.slf4j.Slf4j;
//import org.aspectj.lang.ProceedingJoinPoint;
//import org.aspectj.lang.Signature;
//import org.aspectj.lang.annotation.*;
//import org.aspectj.lang.reflect.MethodSignature;
//import org.springframework.beans.factory.annotation.Autowired;
//import org.springframework.core.annotation.Order;
//import org.springframework.stereotype.Component;
//import org.springframework.web.context.request.RequestContextHolder;
//import org.springframework.web.context.request.ServletRequestAttributes;
//
//import javax.servlet.http.HttpServletRequest;
//import java.util.Arrays;
//
///**
// * ReqAspect Controller请求切面，获取请求者的各项信息
// *
// * 为什么需要AOP
// * AOP（Aspect Orient Programming）也就是面向切面编程，作为面向对象编程的一种补充，已经成为一种比较成熟的编程方式。其实AOP问世的时间并不太长，AOP和OOP互为补充，面向切面编程将程序运行过程分解成各个切面。
// * AOP专门用于处理系统中分布于各个模块（不同方法）中的交叉关注点的问题，在JavaEE应用中，常常通过AOP来处理一些具有横切性质的系统级服务，如事务管理、安全检查、缓存、对象池管理等，AOP已经成为一种非常常用的解决方案。
// *
// * 使用AspectJ实现AOP
// * AspectJ是一个基于Java语言的AOP框架，提供了强大的AOP功能，其他很多AOP框架都借鉴或采纳其中的一些思想。其主要包括两个部分：一个部分定义了如何表达、定义AOP编程中的语法规范，通过这套语法规范，可以方便地用AOP来解决Java语言中存在的交叉关注点的问题；另一个部分是工具部分，包括编译、调试工具等。
// *
// * AOP实现可分为两类：
// * 静态AOP实现: AOP框架在编译阶段对程序进行修改，即实现对目标类的增强，生成静态的AOP代理类，以AspectJ为代表。
// * 动态AOP实现: AOP框架在运行阶段动态生成AOP代理，以实现对目标对象的增强，以Spring AOP为代表。
// * 一般来说，静态AOP实现具有较好的性能，但需要使用特殊的编译器。动态AOP实现是纯Java实现，因此无须特殊的编译器，但是通常性能略差。
// *
// * AOP的基本概念
// * 关于面向切面编程的一些术语：
// * 切面（Aspect）: 切面用于组织多个Advice，Advice放在切面中定义。
// * 连接点（Joinpoint）: 程序执行过程中明确的点，如方法的调用，或者异常的抛出。在Spring AOP中，连接点总是方法的调用。
// * 增强处理（Advice）: AOP框架在特定的切入点执行的增强处理。处理有"around"、"before"和"after"等类型
// * 切入点（Pointcut）: 可以插入增强处理的连接点。简而言之，当某个连接点满足指定要求时，该连接点将被添加增强处理，该连接点也就变成了切入点。
// * Spring的AOP支持
// * Spring中的AOP代理由Spring的IoC容器负责生成、管理，其依赖关系也由IoC容器负责管理。
// * 为了在应用中使用@AspectJ支持，Spring需要添加三个库：
// * aspectjweaver.jar
// * aspectjrt.jar
// * aopalliance.jar
// *
// * @author lzh
// * @date 2020/9/21 - 10:13
// */
//@Aspect
//@Order(1)
//// 控制多个Aspect的执行顺序，越小越先执行
//@Component
//@Slf4j
//public class ReqLogAspect {
//
//    @Autowired
//    private AsyncAllService asyncAllService;
//    @Autowired
//    private IdGenerator idGenerator;
//
//    /**
//     * 定义 切点
//     * 切点可以是 package路径 或 自定义注解路径
//     * package路径 如  execution(* cn.ath.knowwikibackend.rest..*.*(..))
//     * 自定义注解路径 如 @annoation(cn.ath.knowwikibackend.anno.MyAnnotation)
//     */
//    @Pointcut("execution(* cn.ath.knowwikibackend.rest..*.*(..))")
//    public void myPointCut(){}
//
//    //环绕通知(可以获得请求前和响应后的数据)
////    切面为cn.ath.knowwikibackend.rest包下的类
////    @Around(value = "execution(* cn.ath.knowwikibackend.rest..*.*(..))")
//    @Around(value = "myPointCut()") //可以指向切点方法 或者 直接指向切点的路径
//    public Object handleControllerMethod(ProceedingJoinPoint proceedingJoinPoint) throws Throwable {
//        log.info("ReqAspect->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
//        long startTime = System.currentTimeMillis();
//        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
//        OperationLogIndex logEntity = new OperationLogIndex();
//        logEntity.setId(idGenerator.snowflakeId());
//        logEntity.setGmtCreate(DateTime.now().toString());
//        if (attributes != null) {
//            HttpServletRequest request = attributes.getRequest();
//            String ip = Ip2regionUtil.getIp(request);
//            logEntity.setReqIp(ip);
//            String addr = Ip2regionUtil.getAddr(ip);
//            logEntity.setReqLocation(addr);
//            log.info("请求Port:{}", request.getRemotePort());
//            log.info("请求URI:{}", request.getRequestURI());
//            String token = request.getHeader("Authorization");
//            log.info("ReqHeader-Authorization:{}", token);
//            String account = "";
//            if (StrUtil.isNotBlank(token)) {
//                logEntity.setReqToken(token);
////                account = (String) StpUtil.getLoginIdByToken(token);
////                log.info("操作人:{}", account);
//            }
//            logEntity.setAccount(account);
//            logEntity.setReqUrl(request.getRequestURI());
//        }
//        Signature signature = proceedingJoinPoint.getSignature();
//        log.info("请求目标类:{}", signature);
//        if (signature instanceof MethodSignature) {
//            MethodSignature methodSignature = (MethodSignature) signature;
//            //获取方法上加的自定义注解 @BusinessLogic 的属性
//            BusinessLogic businessLogic = methodSignature.getMethod().getAnnotation(BusinessLogic.class);
//            if (businessLogic != null) {
//                logEntity.setBusinessUnit(businessLogic.businessUnit().getBusinessDesc());
//                logEntity.setBusinessMethod(businessLogic.value());
//                log.info("业务操作:{}", businessLogic.value());
//                log.info("业务模块:{}", businessLogic.businessUnit().getBusinessDesc());
//            }
//        }
//        // 获取请求入参
//        Object[] args = proceedingJoinPoint.getArgs();
//        Arrays.stream(args).forEach(arg -> log.info("请求的 arg item is {}", arg));
//        // 获取响应
//        Object response = proceedingJoinPoint.proceed();
//        long endTime = System.currentTimeMillis();
//        log.info("Around请求耗时:{}ms", (endTime - startTime));
//        //异步保存操作日志
//        asyncAllService.asyncSaveOperationLog(logEntity);
//
//
//
//        log.info("ReqAspect-<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<");
//        return response;
//    }
//
//
//    @Before(value = "myPointCut()")
//    public void before() throws Throwable{
//        log.info("测试AOP 前置通知");
//    }
//
//    @After(value = "myPointCut()")
//    public void after() throws Throwable{
//        log.info("测试AOP 后置通知");
//    }
//
//
//}
