package cn.demo;

import cn.demo.biz.AptBomService;
import cn.demo.biz.AptInstService;
import cn.demo.biz.AsyncService;
import cn.demo.biz.dto.BomDTO;
import cn.demo.biz.dto.BomPointDTO;
import cn.demo.biz.dto.BomTopicDTO;
import cn.demo.biz.dto.InstDTO;
import cn.demo.orm.dao.*;
import cn.demo.orm.po.*;
import cn.demo.sys.JacksonUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.StrUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.fusesource.mqtt.client.*;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;

@Slf4j
@SpringBootTest
public class Tests {

    @Resource
    private AppBomMapper appBomMapper;
    @Resource
    private AppMetaMapper appMetaMapper;
    @Resource
    private PointMetaMapper pointMetaMapper;
    @Resource
    private TopicMetaMapper topicMetaMapper;

    @Resource
    private AppInstMapper appInstMapper;
    @Resource
    private PointInstMapper pointInstMapper;
    @Resource
    private TopicManageMapper topicManageMapper;

    @Autowired
    private AptBomService aptBomService;
    @Autowired
    private AsyncService asyncService;
    @Autowired
    private AptInstService aptInstService;

    @Resource
    private TopicMsgAllMapper msgAllMapper;

    @Test
    void r1(){
        List<TopicMsgAll> topicMsgAlls = msgAllMapper.listByWhere(TopicMsgAll.builder()
                        .startTime(1688205428429L)
                        .endTime(1688205987262L)
                        .topic("up/cck/valu/cck1")
                .build());

        System.out.println(topicMsgAlls);
    }


    @Test
    void w1() throws ExecutionException, InterruptedException {
        InstDTO aptInstByAppId = aptInstService.getAptInstByAppId(1L);
        System.out.println(JacksonUtil.obj2StringPretty(aptInstByAppId));
    }

    @Test
    void k0() {
        String s1 = "down/switch/status/[serno]";
        String s2 = "sch1688027223829";

        String res = StrUtil.replace(s1, "[serno]", s2);
        System.out.println(res);
    }

    @Test
    void k1() throws ExecutionException, InterruptedException {
        BomDTO oneBom = aptBomService.getOneBomByAppMetaId(7L);
        System.out.println(JacksonUtil.obj2StringPretty(oneBom));


        //build appInst
        AppInst appInst = AppInst.builder()
                .sname(oneBom.getName() + System.currentTimeMillis())
                .sno(oneBom.getSnt() + System.currentTimeMillis())
                .ts(System.currentTimeMillis())
                .appMetaId(oneBom.getId()).build();
        appInstMapper.insertBatch(ListUtil.toList(appInst));
        System.out.println(appInst);

        //build pointInst and topicInst
        List<BomPointDTO> bomPointList = oneBom.getBomPointList();
        for (BomPointDTO bomPoint : bomPointList) {
            generatePointInstItem(bomPoint, appInst.getId());
        }

    }

    private void generatePointInstItem(BomPointDTO bomPoint, Long appInstId) {

        List<PointInst> pointInsts = new ArrayList<>();
        for (long i = 0; i < bomPoint.getCost(); i++) {
            //save pointInst
            PointInst pointInst = PointInst.builder()
                    .appInstId(appInstId)
                    .pointMetaId(bomPoint.getId())
                    .serno(bomPoint.getSer() + i)
                    .ts(System.currentTimeMillis())
                    .build();
            pointInsts.add(pointInst);
        }
        pointInstMapper.insertBatch(pointInsts);


        asyncService.generateTopicInstList(pointInsts,bomPoint,appInstId);



    }


    @Test
    void s1() {
        AppMeta icd = AppMeta.builder()
                .name("iot-meta-demo")
                .remark("iot-meta-demo-design")
                .snt("imd")
                .ts(System.currentTimeMillis())
                .build();
        appMetaMapper.insertOne(icd);

        System.out.println(icd);


    }

    @Test
    void s2() {
        List<PointMeta> pre = new ArrayList<>();
        pre.add(PointMeta.builder()
                .name("led")
                .unit("none")
                .remark("0-灭；1-亮")
                .ser("LED")
                .ts(System.currentTimeMillis())
                .cost(1L)
                .build());
        pre.add(PointMeta.builder()
                .name("temperature")
                .unit("℃")
                .remark("温度传感器，值范围 正负100")
                .ser("TEMP")
                .ts(System.currentTimeMillis())
                .cost(1L)
                .build());

        pointMetaMapper.insertBatch(pre);
    }

    @Test
    void s4() {
        topicMetaMapper.insertOne(TopicMeta.builder()
                .pointMetaId(4L) //LED
                .topic("down/LED/status/[serno]")
                .exampleData("0")
                .remark("云端控制开关LED: 0-灭；1-亮. serno是点位实例的唯一标号")
                .directType("down")
                .createTs(System.currentTimeMillis())
                .build());

        topicMetaMapper.insertOne(TopicMeta.builder()
                .pointMetaId(4L) //LED
                .topic("up/LED/status/[serno]")
                .exampleData("0")
                .remark("硬件主动上报LED状态: 0-灭；1-亮. serno是点位实例的唯一标号")
                .directType("up")
                .createTs(System.currentTimeMillis())
                .build());

        topicMetaMapper.insertOne(TopicMeta.builder()
                .pointMetaId(5L) //TEMP
                .topic("up/TEMP/status/[serno]")
                .exampleData("0")
                .remark("温度传感器主动上报: 值范围 正负100. serno是点位实例的唯一标号")
                .directType("up")
                .createTs(System.currentTimeMillis())
                .build());


    }

    @Test
    void s3() {
        List<AppBom> pre = new ArrayList<>();

        pre.add(AppBom.builder()
                .appMetaId(3L)
                .pointMetaId(4L)
                .ts(System.currentTimeMillis())
                .build());
        pre.add(AppBom.builder()
                .appMetaId(3L)
                .pointMetaId(5L)
                .ts(System.currentTimeMillis())
                .build());

        appBomMapper.insertBatch(pre);
    }


    @Test
    void ttt() {
        int pageNo = 2;  //pageNo从1开始
        int pageSize = 5;
        PageHelper.startPage(pageNo, pageSize);
        List<TopicManage> list = topicManageMapper.listByWhere(TopicManage.builder()
                .directType("up")
                .build());
        PageInfo<TopicManage> topicManagePageInfo = new PageInfo<>(list);
        System.out.println(JacksonUtil.obj2StringPretty(topicManagePageInfo));
    }

    @Test
    void tt3() {
        //直接limit 分页,不进行count计算
        //LIMIT ?, ?
        int pageNo = 2;  //pageNo从1开始
        int pageSize = 5;
        PageHelper.startPage(pageNo, pageSize, false);
        List<TopicManage> list = topicManageMapper.listByWhere(TopicManage.builder()
                .directType("up")
                .build());
        PageInfo<TopicManage> topicManagePageInfo = new PageInfo<>(list);
        System.out.println(JacksonUtil.obj2StringPretty(topicManagePageInfo));
    }

    @Test
    void tt2() {
        //带 order by 排序字段的分页
        //order by id LIMIT ?
        int pageNo = 1;  //pageNo从1开始
        int pageSize = 5;
        PageHelper.startPage(pageNo, pageSize, "id asc"); //id升序
        List<TopicManage> list = topicManageMapper.listByWhere(TopicManage.builder()
                .directType("up")
                .build());
        PageInfo<TopicManage> topicManagePageInfo = new PageInfo<>(list);
        System.out.println(JacksonUtil.obj2StringPretty(topicManagePageInfo));
    }

    @Test
    void tt1() {
        //带 order by 排序字段的分页
        //order by id LIMIT ?
        int pageNo = 1;  //pageNo从1开始
        int pageSize = 5;
        PageHelper.startPage(pageNo, pageSize, "id desc"); //id降序
        List<TopicManage> list = topicManageMapper.listByWhere(TopicManage.builder()
                .directType("up")
                .build());
        PageInfo<TopicManage> topicManagePageInfo = new PageInfo<>(list);
        System.out.println(JacksonUtil.obj2StringPretty(topicManagePageInfo));
    }


    @Test
    void test() {
        topicManageMapper.insertOne(TopicManage.builder()
                .createTs(new Date().getTime())
                .updateTs(new Date().getTime())
                .remark("门禁上报")
                .topic("up/door/7")
                .exampleData("0")
                .directType("up")
                .enableFlag(0)
                .build());
    }

    @Test
    void t1() throws Exception {
        // Create a new MQTT connection to the broker.  We are not setting the client ID.  The broker will pick one for us.
        System.out.println("Connecting to Artemis using MQTT");
        MQTT mqtt = new MQTT();
        mqtt.setHost("tcp://localhost:1883");
        mqtt.setPassword("admin");
        mqtt.setUserName("admin");
        BlockingConnection connection = mqtt.blockingConnection();
        connection.connect();
        System.out.println("Connected to Artemis");

        // Subscribe to topics
        Topic[] topics = {new Topic("mqtt/example/publish", QoS.AT_LEAST_ONCE),
                new Topic("test/#", QoS.EXACTLY_ONCE),
                new Topic("foo/+/bar", QoS.AT_LEAST_ONCE)};
        connection.subscribe(topics);
        System.out.println("Subscribed to topics.");

        // Publish Messages
        String payload1 = "This is message 1";
        String payload2 = "This is message 2";
        String payload3 = "This is message 3";

        connection.publish("mqtt/example/publish", payload1.getBytes(), QoS.AT_LEAST_ONCE, false);
        connection.publish("test/test", payload2.getBytes(), QoS.AT_MOST_ONCE, false);
        connection.publish("foo/1/bar", payload3.getBytes(), QoS.AT_MOST_ONCE, false);
        System.out.println("Sent messages.");

        Message message1 = connection.receive(5, TimeUnit.SECONDS);
        Message message2 = connection.receive(5, TimeUnit.SECONDS);
        Message message3 = connection.receive(5, TimeUnit.SECONDS);
        System.out.println("Received messages.");

        System.out.println(new String(message1.getPayload()));
        System.out.println(new String(message2.getPayload()));
        System.out.println(new String(message3.getPayload()));
    }
}
